
print("---- Amphibious Assault Ship mod ----");

local enabledLTMod = false;
function detectLTMod()
	local modList = Modding.GetInstalledMods();
	for k, v in pairs(modList) do 
		if( v ~= nil and v.Enabled and v.Name == "LightTouch" ) then
			enabledLTMod = true;
		end
	end
	if(enabledLTMod) then print("checking mods: LightTouch Mod is enabled");
	else print("checking mods: LightTouch Mod is disabled"); end
end
detectLTMod();

function InputHandler( uiMsg, wParam, lParam )
	local interfaceMode = UI.GetInterfaceMode();
	if((interfaceMode == InterfaceModeTypes.INTERFACEMODE_SELECTION and uiMsg == MouseEvents.RButtonUp ) or 
				(interfaceMode == InterfaceModeTypes.INTERFACEMODE_MOVE_TO and uiMsg == MouseEvents.LButtonUp)) then
		
		local pPlot = Map.GetPlot( UI.GetMouseOverHex() );
		local pSelUnit = UI.GetHeadSelectedUnit();
		
		if(pPlot~=nil and pSelUnit~=nil)then
		
			--check that unit is of correct class
			local unitType = pSelUnit:GetUnitType();
			local unitSpType = pSelUnit:GetSpecialUnitType();
			if( unitSpType == GameInfoTypes.SPECIALUNIT_MARINE_EXPEDITIONARY_UNIT ) then
				
				local selUnitPlot = pSelUnit:GetPlot();
				local selUnitX = selUnitPlot:GetX();
				local selUnitY = selUnitPlot:GetY();
				
				local plotX = pPlot:GetX();
				local plotY = pPlot:GetY();				
				local distanceTo = Map.PlotDistance(selUnitX, selUnitY, plotX, plotY);
				local unitMoves = pSelUnit:GetMoves();
				local unitIsCargo = pSelUnit:IsCargo();
				
				local allowLTControl = true;
				if( unitIsCargo and (distanceTo > 1 or unitMoves==0) ) then
					allowLTControl = false;
				end
				local issueLTSleepMission = false;
				--print(distanceTo, pSelUnit:GetMoves(),unitIsCargo, "XY:",selUnitX, selUnitY, "XY:",plotX, plotY);
				
				if ( distanceTo == 1 and unitMoves > 0 ) then
					--check if unit is already loaded on AAS
					if( unitIsCargo ) then
						if( not pPlot:IsWater() ) then	--distance exactly 1 : means unit wants to disembark into adjacent cell
							pSelUnit:PushMission(GameInfoTypes.MISSION_MOVE_TO, plotX, plotY);
							pSelUnit:SetMoves(pSelUnit:MaxMoves() - GameDefines.MOVE_DENOMINATOR);	--set moves to max amount - 1
						else
							--oops, no walking on the water!
							allowLTControl = false;
						end
					else
			
						--check if there's an AAS in target plot	
						playerID = Game.GetActivePlayer();
						local pUnit = nil;
						local unitCount = pPlot:GetNumUnits();
						for i = 0, unitCount - 1, 1 do
							local pFoundUnit = pPlot:GetUnit(i);
							local thisUnitType = pFoundUnit:GetUnitType();
							if (thisUnitType==GameInfoTypes.UNIT_AAS and pFoundUnit:GetOwner()==playerID) then  
								pUnit = pFoundUnit;
								break;
							end
						end
						local canLoad = pUnit~=nil and not pUnit:IsFull();
						
						if(canLoad) then
							local targetUnitState=pUnit:GetActivityType();
							
							--teleport to AAS and drain moves
							pSelUnit:SetXY(plotX, plotY);
							pSelUnit:SetMoves(0);
							
							allowLTControl = false; -- disallow LT from controlling the unit
							issueLTSleepMission = true; -- tell LT to put unit to Sleep next turn
							
							--following piece of code will allow the ship to restore its state, if LightTouch mod is enabled
							if(targetUnitState==ActivityTypes.ACTIVITY_SENTRY or targetUnitState==ActivityTypes.ACTIVITY_SLEEP) then
								local targetMission=targetUnitState==ActivityTypes.ACTIVITY_SENTRY and GameInfoTypes.MISSION_ALERT or GameInfoTypes.MISSION_SLEEP;
								-- this AAS is on alert/sleep. It might have some missions in stack though.
								-- add a mission to restore state as first in stack, with Not Ready Flag set, so event TeleportToHex will only restore the state
								LuaEvents.LTMOD_RemoteControl(pUnit:GetID(), "MISSION", "first", targetMission, {nil}, {"NRF"});
							end
							
						end
										
					end
				end
				
				if(not allowLTControl) then
					LuaEvents.LTMOD_RemoteControl(pSelUnit:GetID(), "BLACKLIST_NOW");	--if LT mod is enabled, make it ignore this unit's current control
					if(issueLTSleepMission) then
						--For some reason, the following line is not necessary. Teleported units will be set to sleep by game engine. Still, better be safe...
						LuaEvents.LTMOD_RemoteControl(pSelUnit:GetID(), "MISSION", "only", GameInfoTypes.MISSION_SLEEP, {nil}, {"NFE", "TF"});
					end
					if(not enabledLTMod and unitIsCargo) then
						pSelUnit:Embark();				--prevent walking on the water
						pSelUnit:SetMoves((pSelUnit:GetMoves() > 0) and pSelUnit:MaxMoves() or 0);	--and set the correct moves for embarked unit
					end
					UI.SetInterfaceMode(InterfaceModeTypes.INTERFACEMODE_SELECTION); --prevents locking in move interface
				end
				Events.ClearHexHighlights();			
				
			end
		end
	end
end
ContextPtr:SetInputHandler( InputHandler );
